package models

import (
	bnet "blackforestbytes.com/badennet"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/bsontype"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"gogs.mikescher.com/BlackForestBytes/goext/exerr"
)

type EntityID interface {
	ObjID() (primitive.ObjectID, error)
	String() string
	MarshalBSONValue() (bsontype.Type, []byte, error)
	AsAny() AnyID
}

// ------------------------------------------------------------

type OrgUserID string

func (i OrgUserID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal OrgUserID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i OrgUserID) String() string {
	return string(i)
}

func (i OrgUserID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i OrgUserID) AsAny() AnyID {
	return AnyID(i)
}

func NewOrgUserID() OrgUserID {
	return OrgUserID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type AdminUserID string

func (i AdminUserID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal AdminUserID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i AdminUserID) String() string {
	return string(i)
}

func (i AdminUserID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i AdminUserID) AsAny() AnyID {
	return AnyID(i)
}

func NewAdminUserID() AdminUserID {
	return AdminUserID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type AppUserID string

func (i AppUserID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal AppUserID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i AppUserID) String() string {
	return string(i)
}

func (i AppUserID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i AppUserID) AsAny() AnyID {
	return AnyID(i)
}

func NewAppUserID() AppUserID {
	return AppUserID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type PostID string

func (i PostID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal PostID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i PostID) String() string {
	return string(i)
}

func (i PostID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i PostID) AsAny() AnyID {
	return AnyID(i)
}

func NewPostID() PostID {
	return PostID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type CrawlerID string

func (i CrawlerID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal CrawlerID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i CrawlerID) String() string {
	return string(i)
}

func (i CrawlerID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i CrawlerID) AsAny() AnyID {
	return AnyID(i)
}

func NewCrawlerID() CrawlerID {
	return CrawlerID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type CrawlerExecutionID string

func (i CrawlerExecutionID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal CrawlerExecutionID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i CrawlerExecutionID) String() string {
	return string(i)
}

func (i CrawlerExecutionID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i CrawlerExecutionID) AsAny() AnyID {
	return AnyID(i)
}

func NewCrawlerExecutionID() CrawlerExecutionID {
	return CrawlerExecutionID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type CrawlerLogID string

func (i CrawlerLogID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal CrawlerLogID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i CrawlerLogID) String() string {
	return string(i)
}

func (i CrawlerLogID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i CrawlerLogID) AsAny() AnyID {
	return AnyID(i)
}

func NewCrawlerLogID() CrawlerLogID {
	return CrawlerLogID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type BlobID string

func (i BlobID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal BlobID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i BlobID) String() string {
	return string(i)
}

func (i BlobID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i BlobID) AsAny() AnyID {
	return AnyID(i)
}

func NewBlobID() BlobID {
	return BlobID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type ConversationNoteID string

func (i ConversationNoteID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal ConversationNoteID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i ConversationNoteID) String() string {
	return string(i)
}

func (i ConversationNoteID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i ConversationNoteID) AsAny() AnyID {
	return AnyID(i)
}

func NewConversationNoteID() ConversationNoteID {
	return ConversationNoteID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type PasswordResetTokenID string

func (i PasswordResetTokenID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal PasswordResetTokenID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i PasswordResetTokenID) String() string {
	return string(i)
}

func (i PasswordResetTokenID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i PasswordResetTokenID) AsAny() AnyID {
	return AnyID(i)
}

func NewPasswordResetTokenID() PasswordResetTokenID {
	return PasswordResetTokenID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type OrganizationID string

func (i OrganizationID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal OrganizationID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i OrganizationID) String() string {
	return string(i)
}

func (i OrganizationID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i OrganizationID) AsAny() AnyID {
	return AnyID(i)
}

func NewOrganizationID() OrganizationID {
	return OrganizationID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type GroupID string

func (i GroupID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal GroupID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i GroupID) String() string {
	return string(i)
}

func (i GroupID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i GroupID) AsAny() AnyID {
	return AnyID(i)
}

func NewGroupID() GroupID {
	return GroupID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type FAPICallID string

func (i FAPICallID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal FAPICallID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i FAPICallID) String() string {
	return string(i)
}

func (i FAPICallID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i FAPICallID) AsAny() AnyID {
	return AnyID(i)
}

func NewFAPICallID() FAPICallID {
	return FAPICallID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type SolseitAPICallID string

func (i SolseitAPICallID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal SolseitAPICallID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i SolseitAPICallID) String() string {
	return string(i)
}

func (i SolseitAPICallID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i SolseitAPICallID) AsAny() AnyID {
	return AnyID(i)
}

func NewSolseitAPICallID() SolseitAPICallID {
	return SolseitAPICallID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type MigrationID string

func (i MigrationID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal MigrationID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i MigrationID) String() string {
	return string(i)
}

func (i MigrationID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i MigrationID) AsAny() AnyID {
	return AnyID(i)
}

func NewMigrationID() MigrationID {
	return MigrationID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type EventID string

func (i EventID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal EventID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i EventID) String() string {
	return string(i)
}

func (i EventID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i EventID) AsAny() AnyID {
	return AnyID(i)
}

func NewEventID() EventID {
	return EventID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type SolseitMappingID string

func (i SolseitMappingID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal SolseitMappingID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i SolseitMappingID) String() string {
	return string(i)
}

func (i SolseitMappingID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i SolseitMappingID) AsAny() AnyID {
	return AnyID(i)
}

func NewSolseitMappingID() SolseitMappingID {
	return SolseitMappingID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type SolseitCommunityID string

func (i SolseitCommunityID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal SolseitCommunityID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i SolseitCommunityID) String() string {
	return string(i)
}

func (i SolseitCommunityID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i SolseitCommunityID) AsAny() AnyID {
	return AnyID(i)
}

func NewSolseitCommunityID() SolseitCommunityID {
	return SolseitCommunityID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type JobLogID string

func (i JobLogID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal JobLogID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i JobLogID) String() string {
	return string(i)
}

func (i JobLogID) Valid() bool {
	objId, err := primitive.ObjectIDFromHex(string(i))
	return err == nil && objId != primitive.ObjectID{}
}

func (i JobLogID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i JobLogID) AsAny() AnyID {
	return AnyID(i)
}

func NewJobLogID() JobLogID {
	return JobLogID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type JobExecutionID string

func (i JobExecutionID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal JobExecutionID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i JobExecutionID) String() string {
	return string(i)
}

func (i JobExecutionID) Valid() bool {
	objId, err := primitive.ObjectIDFromHex(string(i))
	return err == nil && objId != primitive.ObjectID{}
}

func (i JobExecutionID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i JobExecutionID) AsAny() AnyID {
	return AnyID(i)
}

func NewJobExecutionID() JobExecutionID {
	return JobExecutionID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type SubscriptionID string

func (i SubscriptionID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal SubscriptionID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i SubscriptionID) String() string {
	return string(i)
}

func (i SubscriptionID) Valid() bool {
	objId, err := primitive.ObjectIDFromHex(string(i))
	return err == nil && objId != primitive.ObjectID{}
}

func (i SubscriptionID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i SubscriptionID) AsAny() AnyID {
	return AnyID(i)
}

func NewSubscriptionID() SubscriptionID {
	return SubscriptionID(primitive.NewObjectID().Hex())
}

// ------------------------------------------------------------

type AnyID string

func (i AnyID) MarshalBSONValue() (bsontype.Type, []byte, error) {
	if objId, err := primitive.ObjectIDFromHex(string(i)); err == nil {
		return bson.MarshalValue(objId)
	} else {
		return 0, nil, exerr.New(bnet.ErrMarshalBSON, "Failed to marshal AnyID("+i.String()+") to ObjectId").Str("value", string(i)).Type("type", i).Build()
	}
}

func (i AnyID) String() string {
	return string(i)
}

func (i AnyID) ObjID() (primitive.ObjectID, error) {
	return primitive.ObjectIDFromHex(string(i))
}

func (i AnyID) AsAny() AnyID {
	return i
}

func (i AnyID) AsOrgUserID() OrgUserID {
	return OrgUserID(i)
}

func (i AnyID) AsGroupID() GroupID {
	return GroupID(i)
}

func (i AnyID) AsEventID() EventID {
	return EventID(i)
}
